module PC.View exposing (pc_view)

import Html                         exposing (Html, button, div, h1, p, text)
import Html.Attributes  as HAttr    exposing (class, classList, value)
import Html.Events      as HEvent   exposing (onClick, onInput)
import Html.Lazy                    exposing (lazy)
import Array                        exposing (Array)
import Json.Decode      as JD

import PC.Types                     exposing (..)
import PC.UActions                  exposing (..)
import PC.Helpers                   exposing (..)

pc_view : PC_Model -> Html PC_Msg
pc_view model = div []
    [ div 
        [ class "grid-fullwidth", class "grid-addressbus" ] 
        [ lazy viewAddressBus model ]
    , lazy viewRam model
    , lazy viewCu  model
    , lazy viewAlu model
    , div 
        [ class "grid-fullwidth", class "grid-databus" ] 
        [ lazy viewDataBus    model ]
    ]

-- #####################################################################
-- #####################################################################



-- #####################################################################
-- RAM
-- #####################################################################

viewRam : PC_Model -> Html PC_Msg
viewRam model =
    div [ class "section", class "ram" ]
        [ h1  [ class "header" ] [ text "RAM" ]
        , div [ class "arrow", class "down", class "top"] [ div [] [] , div [] [] ]
        , div [ class "scroller" ]
            [ Html.table [ class "" ]
                [ Html.thead [ class "head" ]
                    [ Html.tr []
                        [ Html.th [ class "address" ] [ text "Addr" ]
                        , Html.th [] [ text "Value" ]
                        , Html.th [] [ text "Comment" ]
                        ]
                    ]
                , lazy viewRamContent model
                ]
            ]
        , drawArrow Bottom Down UA_Ram2DataBus "Ram -> DataBus"
        , drawArrow Bottom Up   UA_DataBus2Ram "DataBus -> Ram"
        ]


viewRamContent : PC_Model -> Html PC_Msg
viewRamContent model =
    let
        --indexedRam =
        --    List.indexedMap Tuple.pair <| Array.toList model.pc.ram

        ram2table : Int -> (Int, String) -> Html PC_Msg
        ram2table id entry =
            let
                (val,comment) = entry

                (instruction, address) = seperateInstructionsEntry val
            in
            Html.tr
                [ classList [ ( "current", id == model.pc.addressBus ) ] ]
                [ Html.td [ class "num" ] [ text (addLeadingZero id 3) ]
                , Html.td [ class "num " ] 
                    [ Html.input 
                        [ HAttr.type_ "number"
                        , HAttr.value <| addLeadingZero instruction 3
                        , onInput <| PM_F_RamEditInstr id
                        , class "ram-entry"
                        , class "instruction"
                        ]
                        []
                    , Html.input 
                        [ HAttr.type_ "number"
                        , HAttr.value <| addLeadingZero address 5
                        , onInput <| PM_F_RamEditAddress id
                        , class "ram-entry"
                        , class "address"
                        ]
                        []
                    ]
                , Html.td[ class "comment" ]
                    [ Html.input
                        [ HAttr.type_ "text"
                        , HAttr.value comment
                        , onInput <| PM_F_RamEditComment id ]
                        []
                    ]
                ]
    in
    Html.tbody [] 
        ((Array.toList <| Array.indexedMap ram2table model.pc.ram)
        ++
        [ Html.tr [] 
            [ Html.td [] []
            , Html.td [] 
                [ button 
                    [ onClick PM_B_RamAddBelow  ] 
                    [ text "Add Entry"] 
                ]
            , Html.td [][]
            ]
        ]
        )

-- #####################################################################
-- #####################################################################




-- #####################################################################
-- Control Unit
-- #####################################################################

viewCu : PC_Model -> Html PC_Msg
viewCu model =
    let
        ( instrRegInst, instrRegAddr ) = seperateInstructionsEntry model.pc.instructionReg
    in
    div [ class "section", class "cu" ]
        [ drawArrow Top Up UA_InstructionReg2AddressBus  "InstR -> AddrB"
        , drawArrow Top Up UA_ProgrammCounter2AddressBus "ProgCounter -> AddrB"
        --, div [ class "arrow", class "up", class "top"] 
        --    [ div [ class "button" ] 
        --        [ Html.a [ onClick (PM_ManualStep ActInstructionReg2AddressBus)] [ text "InstR -> AddrB" ]]
        --    , div [] []
        --    ]
        --, div [ class "arrow", class "up", class "top", class "arrow2"] 
        --    [ div [ class "button" ] 
        --        [ Html.a [ onClick (PM_ManualStep ActProgrammCounter2AddressBus)] [ text "ProgCounter -> AddrB" ]]
        --    , div [] []
        --    ]


        , h1 [ class "header" ] [ text "Control Unit" ]
        , p []
            [ div [class "input-row"] 
                [ Html.label [ HAttr.for "cu-progcounter" ] [ text "Programm Counter:" ]
                , button 
                    [ onClick <| PM_ManualStep UA_InstructionReg2ProgrammCounter ]
                    [ text "InstRA -> " ]
                , Html.input
                    [ HAttr.type_ "number"
                    , HAttr.id "cu-progcounter"
                    , value (addLeadingZero model.pc.programmCounter 3)
                    , onInput PM_F_CuProgCounterEdit
                    ] []
                ]

            , div [class "input-row"] 
                [ Html.label [ HAttr.for "cu-instrReg" ] [ text "Instruction Register:" ]
                , div [] 
                    [ Html.input
                        [ HAttr.type_ "number"
                        , HAttr.id "cu-instrReg"
                        , class "instruction"
                        , value (addLeadingZero instrRegInst 3)
                        , onInput PM_F_CuInstrRegEditInstr
                        ] []
                    , Html.input
                        [ HAttr.type_ "number"
                        , HAttr.id "cu-instrReg"
                        , class "address"
                        , value (addLeadingZero instrRegAddr 5)
                        , onInput PM_F_CuInstrRegEditAddr
                        ] []
                    ]
                ]

            , div [class "input-row"] 
                [ Html.label [ HAttr.for "cu-uCounter" ] [ text "µCode Counter:" ]
                , button 
                    [ onClick <| PM_ManualStep UA_InstructionReg2UCounter
                    ]
                    [ text "IntrRI ->" ]
                , Html.input
                    [ HAttr.type_ "number"
                    , HAttr.id "cu-uCounter"
                    , value (addLeadingZero model.pc.uCounter 4)
                    , onInput PM_F_CuUCounterEdit
                    ] []
                ]

            ]
        , div [ class "scroller" ]
            [ viewCuUCode model
            ]
        --, div [ class "arrow", class "up"] 
        --    [ div [ class "button" ] 
        --        [ Html.a [ onClick (MsgManualStep ActDataBus2InstructionReg) ] [ text "DB -> InstR" ]]
        --    , div [] []
        --    ]
        , drawArrow Bottom Up UA_DataBus2InstructionReg "DB -> InstR"
        ]


viewCuUCode : PC_Model -> Html PC_Msg
viewCuUCode model =
    Html.table []
        [ Html.thead [ class "head" ]
            [ Html.tr []
                [ Html.th [ class "address" ] [ text "µCounter" ]
                , Html.th [] [ text "µCode" ]
                ]
            ]
        , lazy viewCuUCodeContent model
        ]


viewCuUCodeContent : PC_Model -> Html PC_Msg
viewCuUCodeContent model =
    let
        list2table : Int -> UAction -> Html PC_Msg
        list2table id code =
            Html.tr
                [ classList [ ( "current", id == model.pc.uCounter ), ("empty", code == UA_Nothing) ] ]
                [ Html.td [ class "num" ] [ text (addLeadingZero id 4) ]
                , Html.td [] [ viewCuInstrSelect id code ]
                ]

    in
    Html.tbody [] 
        ((Array.toList <| Array.indexedMap list2table model.uCodes)
        ++ 
        [ Html.tr [] 
            [ Html.td [] []
            , Html.td [] 
                [ button
                    [ onClick PM_B_CuAddBelow  ] 
                    [ text "Add Entry"] 
                ]
            ]
        ]
        )

viewCuInstrSelect : Int -> UAction -> Html PC_Msg
viewCuInstrSelect id current =
    let
        info2option uCode = 
                Html.option 
                    [ HAttr.selected (uCode.variant == current)
                    , HAttr.value     uCode.id      ] 
                    [ text            uCode.label   ]
        listOptions = List.map info2option uCodes
    in
    Html.select
        [ HEvent.on "change" ( JD.map (PM_F_CuEditAction id) selectCuValueDecoder) ]
        listOptions

viewInstrEntry : Int -> Html PC_Msg
viewInstrEntry i =
    let
        (instruction, address) = seperateInstructionsEntry i
    in
    text ( (addLeadingZero instruction 3) ++ "  " ++ (addLeadingZero address 5) )

-- #####################################################################
-- #####################################################################




-- #####################################################################
-- ALU
-- #####################################################################


viewAlu : PC_Model -> Html PC_Msg
viewAlu model =
    div [ class "section", class "alu" ]
        [ h1 [ class "header" ] [ text "ALU" ]
        , p [] 
            [ Html.label [ HAttr.for "alu-accumulator" ] [ text "Accumulator:" ]
            , Html.input
                [ HAttr.type_ "number"
                , HAttr.id "alu-accumulator"
                , value (addLeadingZero model.pc.accumulator 8)
                , onInput PM_F_AluEdit
                ] []
            ]
        , p [] 
            [ button [ onClick <| PM_ManualStep UA_AccumulatorIncrement ] [ text "Acc ++" ]
            , button [ onClick <| PM_ManualStep UA_AccumulatorDecrement ] [ text "Acc --" ]
            ]
        , drawArrow Bottom Up UA_DataBus2Accumulator   "DB -> ALU"
        , drawArrow Bottom Down UA_Accumulator2DataBus "ALU -> DB"
        --, div [ class "arrow", class "up"] 
        --    [ div [ class "button" ] 
        --        [ Html.a [ onClick (MsgManualStep ActDataBus2Accumulator)] [ text "DB -> ALU" ]]
        --    , div [] []
        --    ]
        --, div [ class "arrow", class "down"] 
        --    [ div [ class "button" ] 
        --        [ Html.a [ onClick (MsgManualStep ActAccumulator2DataBus)] [ text "ALU -> DB" ]]
        --    , div [] []
        --    ]
        ]

-- #####################################################################
-- Busses
-- #####################################################################

viewDataBus : PC_Model -> Html PC_Msg
viewDataBus model =
    div [ class "databus" ]
        [ Html.span [ class "label" ] [ text "Databus" ]
        , Html.span [] 
            [ Html.input 
                [ HAttr.type_ "number"
                , value (addLeadingZero model.pc.dataBus 8)
                , onInput PM_F_EditDataBus
                ] []
            ]
        ]


viewAddressBus : PC_Model -> Html PC_Msg
viewAddressBus model =
    div [ class "addressbus" ]
        [ Html.span [ class "label" ] [ text "Addressbus" ]
        , Html.span [] 
            [ Html.input 
                [ HAttr.type_ "number"
                , value (addLeadingZero model.pc.addressBus 3)
                , onInput PM_F_EditAddressBus
                ] []
            ]
        ]


-- #####################################################################
-- #####################################################################



-- #####################################################################
-- Helpers
-- #####################################################################

type Arrow_Pos = Top | Bottom
type Arrow_Dir = Up  | Down

drawArrow : Arrow_Pos -> Arrow_Dir -> UAction -> String -> Html PC_Msg
drawArrow pos dir act str =
    div 
        [ HAttr.classList
            [ ("arrow", True)
            , ("top",   pos == Top)
            , ("up",    dir == Up)
            , ("down",  dir == Down)]
        ]
        [ div [ class "button" ] 
            [ Html.a 
                [ onClick (PM_ManualStep act) ] 
                [ text str ]
            ]
        , div [] []
        ]


selectCuValueDecoder : JD.Decoder String
selectCuValueDecoder =
    JD.field "target" ( JD.field "value" JD.string)


-- #####################################################################
-- #####################################################################
-- Done.


